@@ -1,35 +1,16 @@
 /*
  * crt1.c
+ * This file has no copyright assigned and is placed in the Public Domain.
+ * This file is a part of the mingw-runtime package.
+ * No warranty is given; refer to the file DISCLAIMER within the package.
  *
  * Source code for the startup proceedures used by all programs. This code
  * is compiled to make crt1.o, which should be located in the library path.
  *
- * This code is part of the Mingw32 package.
- *
- * Contributors:
- *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- *  Maintained by Mumit Khan <khan@xraylith.wisc.EDU>
- *
- *  THIS SOFTWARE IS NOT COPYRIGHTED
- *
- *  This source code is offered for use in the public domain. You may
- *  use, modify or distribute it freely.
- *
- *  This code is distributed in the hope that it will be useful but
- *  WITHOUT ANY WARRANTY. ALL WARRENTIES, EXPRESS OR IMPLIED ARE HEREBY
- *  DISCLAMED. This includes but is not limited to warrenties of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * $Revision: 1.5 $
- * $Author: earnie $
- * $Date: 2003/01/06 22:14:18 $
- *
  */
 
-/* Hide the declaration of _fmode with dllimport attribute in stdlib.h.
-   This is not necessary with Mumit Khan's patches to gcc's winnt.c,
-   but those patches are still unofficial.  */
-
+/* Hide the declaration of _fmode with dllimport attribute in stdlib.h to
+   avoid problems with older GCC. */
 #define __IN_MINGW_RUNTIME 
 #include <stdlib.h>
 #include <stdio.h>
@@ -47,7 +28,6 @@
  *       a-good-idea use of include. */
 #include "init.c"
 
-
 extern void _pei386_runtime_relocator (void);
 
 extern int main (int, char **, char **);
@@ -68,7 +48,9 @@ __MINGW_IMPORT void __set_app_type(int);
 /* Override the dllimport'd declarations in stdlib.h */
 #undef _fmode 
 extern int _fmode; 
+#ifdef __MSVCRT__
 extern int* __p__fmode(void); /* To access the dll _fmode */
+#endif
 
 /*
  * Setup the default file handles to have the _CRT_fmode mode, as well as
@@ -77,7 +59,7 @@ extern int* __p__fmode(void); /* To access the dll _fmode */
 extern int _CRT_fmode;
 
 static void
-_mingw32_init_fmode ()
+_mingw32_init_fmode (void)
 {
   /* Don't set the std file mode if the user hasn't set any value for it. */
   if (_CRT_fmode)
@@ -106,8 +88,11 @@ _mingw32_init_fmode ()
     }
 
     /*  Now sync  the dll _fmode to the  one for this .exe.  */
+#ifdef __MSVCRT__
     *__p__fmode() = _fmode;	
-
+#else
+    *_imp___fmode_dll = _fmode;
+#endif
 }
 
 /* This function will be called when a trap occurs. Thanks to Jacob
@@ -175,8 +160,8 @@ _gnu_exception_handler (EXCEPTION_POINTERS * exception_data)
 /*
  * The function mainCRTStartup is the entry point for all console programs.
  */
-static int
-__mingw_CRTStartup ()
+static void  __attribute__((noreturn))
+__mingw_CRTStartup (void)
 {
   int nRet;
 
@@ -205,11 +190,14 @@ __mingw_CRTStartup ()
    * NOTE: DLLs don't do this because that would be rude!
    */
   _mingw32_init_fmode ();
-
   
    /* Adust references to dllimported data that have non-zero offsets.  */
   _pei386_runtime_relocator ();
 
+  /* Align the stack to 16 bytes for the sake of SSE ops in main
+     or in functions inlined into main.  */
+  asm  __volatile__  ("andl $-16, %%esp" : : : "%esp");
+
   /*
    * Call the main function. If the user does not supply one
    * the one in the 'libmingw32.a' library will be linked in, and
@@ -225,21 +213,18 @@ __mingw_CRTStartup ()
   _cexit ();
 
   ExitProcess (nRet);
-
-  return 0;
 }
 
 /*
  * The function mainCRTStartup is the entry point for all console programs.
  */
-int
-mainCRTStartup ()
+void
+mainCRTStartup (void)
 {
 #ifdef __MSVCRT__
   __set_app_type (__CONSOLE_APP);
 #endif
   __mingw_CRTStartup ();
-  return 0;
 }
 
 /*
@@ -247,14 +232,13 @@ mainCRTStartup ()
  * This simply gets rid of the annoying warning about not being able
  * to find WinMainCRTStartup when linking GUI applications.
  */
-int
-WinMainCRTStartup ()
+void
+WinMainCRTStartup (void)
 {
 #ifdef __MSVCRT__
   __set_app_type (__GUI_APP);
 #endif
   __mingw_CRTStartup ();
-return 0;
 }
 
 /*
