@@ -1,32 +1,13 @@
 /*
  * direct.h
+ * This file has no copyright assigned and is placed in the Public Domain.
+ * This file is a part of the mingw-runtime package.
+ * No warranty is given; refer to the file DISCLAIMER within the package.
  *
  * Functions for manipulating paths and directories (included from io.h)
  * plus functions for setting the current drive.
  *
- * This file is part of the Mingw32 package.
- *
- * Contributors:
- *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- *
- *  THIS SOFTWARE IS NOT COPYRIGHTED
- *
- *  This source code is offered for use in the public domain. You may
- *  use, modify or distribute it freely.
- *
- *  This code is distributed in the hope that it will be useful but
- *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
- *  DISCLAIMED. This includes but is not limited to warranties of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * $Revision: 1.4 $
- * $Author: earnie $
- * $Date: 2003/02/21 21:19:51 $
- *
  */
-
-#ifndef __STRICT_ANSI__
-
 #ifndef	_DIRECT_H_
 #define	_DIRECT_H_
 
@@ -90,6 +71,3 @@ _CRTIMP int __cdecl _wrmdir(const wchar_t*);
 #endif	/* Not RC_INVOKED */
 
 #endif	/* Not _DIRECT_H_ */
-
-#endif	/* Not __STRICT_ANSI__ */
-
