@@ -1,36 +1,19 @@
 /*
  * timeb.h
+ * This file has no copyright assigned and is placed in the Public Domain.
+ * This file is a part of the mingw-runtime package.
+ * No warranty is given; refer to the file DISCLAIMER within the package.
  *
  * Support for the UNIX System V ftime system call.
  *
- * This file is part of the Mingw32 package.
- *
- * Contributors:
- *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
- *
- *  THIS SOFTWARE IS NOT COPYRIGHTED
- *
- *  This source code is offered for use in the public domain. You may
- *  use, modify or distribute it freely.
- *
- *  This code is distributed in the hope that it will be useful but
- *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
- *  DISCLAIMED. This includes but is not limited to warranties of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- *
- * $Revision: 1.3 $
- * $Author: dannysmith $
- * $Date: 2001/11/29 04:26:33 $
- *
  */
 
-#ifndef	__STRICT_ANSI__
-
 #ifndef	_TIMEB_H_
 #define	_TIMEB_H_
 
 /* All the headers include this file. */
 #include <_mingw.h>
+#include <sys/types.h>
 
 #ifndef	RC_INVOKED
 
@@ -58,18 +41,30 @@ struct timeb
 };
 #endif
 
+struct __timeb64
+{
+	__time64_t time;
+	short millitm;
+	short timezone;
+	short dstflag;
+};
 
 #ifdef	__cplusplus
 extern "C" {
 #endif
 
 /* TODO: Not tested. */
-void	_ftime (struct _timeb*);
+_CRTIMP void __cdecl	_ftime (struct _timeb*);
 
 #ifndef	_NO_OLDNAMES
-void	ftime (struct timeb*);
+_CRTIMP void __cdecl	ftime (struct timeb*);
 #endif	/* Not _NO_OLDNAMES */
 
+/* This requires newer versions of msvcrt.dll (6.10 or higher).  */ 
+#if __MSVCRT_VERSION__ >= 0x0601
+_CRTIMP void __cdecl	_ftime64 (struct __timeb64*);
+#endif /* __MSVCRT_VERSION__ >= 0x0601 */
+
 #ifdef	__cplusplus
 }
 #endif
@@ -77,6 +72,3 @@ void	ftime (struct timeb*);
 #endif	/* Not RC_INVOKED */
 
 #endif	/* Not _TIMEB_H_ */
-
-#endif	/* Not __STRICT_ANSI__ */
-
